
/* cruelty :) */

/* Copyright (C) 1997 Ian Main
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "imgsink.h"
#include "interface.h"
#include "dillo.h"
#include "nav.h"
#include "browser.h"
#include "menu.h"
#include "misc.h"

#define LOAD_BOOKMARKS 1
#define SAVE_BOOKMARK 2
#define CLOSE_BOOKMARKS 3

/* this #define will cut page title if > 39 chars */
#define TITLE39

/* double quote */
#define D_QUOTE 0x22

/*
 * Forward declarations
 */
static void Bookmarks_load(char *file);
static void Bookmarks_add_to_menu(FILE *fp);
static void Bookmarks_file_op(gint operation, char *title, char *url);
static void Bookmarks_save_to_file(FILE *fp, char *title, char *url);
static char *Bookmarks_search_line(char *line, char *start_text, char *end_text);

/*
 * Local data
 */
static DilloBookmark *bookmarks = NULL;
static gint num_bookmarks;
static gint num_bookmarks_max;


/*
 * Allocate memory and load the bookmarks list
 */
void a_Bookmarks_init(void)
{
   gchar *file;

   num_bookmarks = 0;
   num_bookmarks_max = 16;
   bookmarks = g_new(DilloBookmark, num_bookmarks_max);

   /* Here we load and set the bookmarks */
   file = a_Misc_prepend_user_home(".dillo/bookmarks.html");
   Bookmarks_load(file);
   g_free(file);
}

/*
 * ?
 */
static void Bookmarks_goto_bookmark(GtkWidget *widget, BrowserWindow *bw)
{
   gint i;

   for (i = 0; i < num_bookmarks; i++)
      if (widget == bookmarks[i].menuitem)
         break;
   if (i == num_bookmarks) {
      g_warning("bookmark not found!\n");
      return;
   }
   a_Nav_push(bw, bookmarks[i].url);
}

/*
 * Add a bookmark to the bookmarks menu of a particular browser window
 */
void a_Bookmarks_add_to_menu(BrowserWindow *bw, GtkWidget *menuitem)
{
   gtk_menu_append(GTK_MENU(bw->bookmarks_menu), menuitem);

   /* accelerator goes here */

   gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
                      (GtkSignalFunc) Bookmarks_goto_bookmark, bw);
}

/*
 * ?
 */
static GtkWidget *Bookmarks_insert(const char *title, const char *url)
{
   GtkWidget *menuitem;

   if (title != NULL) {
      menuitem = gtk_menu_item_new_with_label(title);
   } else {
      menuitem = gtk_menu_item_new_with_label(url);
   }
   gtk_widget_show(menuitem);

   a_List_add(bookmarks, num_bookmarks, sizeof(*bookmarks), num_bookmarks_max);
   if (title != NULL) {
      bookmarks[num_bookmarks].title = g_strdup(title);
   } else {
      bookmarks[num_bookmarks].title = g_strdup(url);
   }
   bookmarks[num_bookmarks].url = g_strdup(url);
   bookmarks[num_bookmarks].menuitem = menuitem;
   num_bookmarks++;
   return menuitem;
}

/*
 * Add the new bookmark to bookmarks menu of _all_ browser windows and then
 * write the new bookmark to file
 */
void a_Bookmarks_add(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   gint i;
#ifdef TITLE39   
   gboolean allocated = FALSE;
#endif   
   char *title, *url;
   GtkWidget *menuitem;

   title = menu_popup.info.title;
   url = menu_popup.info.url;

   if (!title && !url) {
      /* find title and url */
      if (bw->nav_stack_size == 0)
         return;

      title = bw->nav_stack[bw->nav_stack_ptr].title;
      url = bw->nav_stack[bw->nav_stack_ptr].url;
   }

   /* do not allow an empty url */
   if (url == NULL)
      return;

   /* If the page has no title, lets use the URL */
   if (strcmp (title, "") == 0)
      title = url;

#ifdef TITLE39
   if (strlen (title) > 39) {
      char buf1[20];
      char buf2[20];
      
      strncpy (buf1, title, 18);
      buf1[18] = '\0';
      strcpy (buf2, title + strlen (title) - 18);
      buf2[18] = '\0';
      title = g_strconcat (buf1, "...", buf2, NULL);
      allocated = TRUE;
   }      
#endif

#ifdef VERBOSE
   g_print("add bookmark: %s %s\n", title, url);
#endif

   /* add bookmark to bookmarks menu of _all_ browser windows. */
   menuitem = Bookmarks_insert(title, url);
   for (i = 0; i < num_bw; i++)
      a_Bookmarks_add_to_menu(browser_window[i], menuitem);

   /* write bookmark to file */
   Bookmarks_file_op(SAVE_BOOKMARK, title, url);

#ifdef TITLE39   
   if (allocated)
      g_free (title);
#endif   
}

/*
 * Never called (the file remains open all the time)  --Jcid
 */
void Bookmarks_close(void)
{
   Bookmarks_file_op(CLOSE_BOOKMARKS, NULL, NULL);
}

/*
 * Load bookmarks from ~/.dillo/bookmarks.html
 */
static void Bookmarks_load(char *file)
{
   Bookmarks_file_op(LOAD_BOOKMARKS, file, NULL);
}

/*
 * Performs operations on the bookmark file..
 * for first call, title is the filename
 */
static void Bookmarks_file_op(gint operation, char *title, char *url)
{
   static FILE *fp;
   static gint init = 1;

   if (init) {
      if (operation == LOAD_BOOKMARKS) {
         if ((fp = fopen(title, "a+")) == NULL)
            g_print("dillo: opening bookmark file %s: %s\n",
                    title, strerror(errno));
      } else {
         printf("erroneous call to bookmark_file_op - init but not LOAD_BOOKMARKS\n");
      }
      init = 0;
   }
   switch (operation) {
   case LOAD_BOOKMARKS:
      Bookmarks_add_to_menu(fp);
      break;

   case SAVE_BOOKMARK:
      Bookmarks_save_to_file(fp, title, url);
      break;

   case CLOSE_BOOKMARKS:
      fclose(fp);
      break;

   default:
      break;
   }
}

/*
 * Save bookmarks to ~/.dillo/bookmarks.html
 */
static void Bookmarks_save_to_file(FILE *fp, char *title, char *url)
{
   char *buf;

   fseek(fp, 0L, SEEK_END);

   /* if there is no title, use url as title */
   if (title != NULL) {
      buf = g_malloc(strlen(title) + strlen(url) + 128);
      /* CP: Adding <li>...</li> in prep for future structuring of file */
      sprintf(buf, "<li><A HREF=%c%s%c>%s</A></li>\n", D_QUOTE, url,
              D_QUOTE, title);
   } else {
      buf = g_malloc(strlen(url) + strlen(url) + 128);
      sprintf(buf, "<li><A HREF=%c%s%c>%s</A></li>\n", D_QUOTE, url,
              D_QUOTE, url);
   }
   fwrite(buf, strlen(buf), 1, fp);
   g_free(buf);
}

/*
 * Load bookmarks 
 */
static void Bookmarks_add_to_menu(FILE *fp)
{
   char *title;
   char *url;
   char buf[4096];
   gint i;
   GtkWidget *menuitem;

   rewind(fp);

   g_print("Loading bookmarks...\n");
   while (1) {
      /* Read a whole line from the file */
      if ((fgets(buf, 4096, fp)) == NULL)
         break;

      /* get url from line */
      if ( !(url = Bookmarks_search_line(buf, "=\"", "\">")) )
         continue;

      /* get title from line */
      if ( !(title = Bookmarks_search_line(buf, "\">", "</")) ){
         g_free(url);
         continue;
      }

      menuitem = Bookmarks_insert(title, url);
      for (i = 0; i < num_bw; i++)
         a_Bookmarks_add_to_menu(browser_window[i], menuitem);
      g_free(url);
      g_free(title);
   }
}

/*
 * Return the text between start_text and end_text in line.
 * ** this function allocates memory for text_return which must be freed! **
 * I really hope there isn't a gtk or one of your html functions 
 * that does this :) I'm sure there must be an html one already..
 * 
 * On error, returns NULL.
 */
static char *
 Bookmarks_search_line(char *line, char *start_text, char *end_text) {
   gint segment_length;
   char *start_index;
   char *end_index;
   char *text_return;

   /* if string is not found, return NULL */
   if ((start_index = strstr(line, start_text)) == NULL)
      return (NULL);

   if ((end_index = strstr(line, end_text)) == NULL)
      return (NULL);

   /* adjustment cause strstr returns the start of the text */
   start_index += strlen(start_text);

   /* find length of text segment */
   segment_length = end_index - start_index;

   text_return = g_malloc((sizeof(char) * segment_length) + 2);

   strncpy(text_return, start_index, segment_length);
   text_return[segment_length] = '\0';

   return (text_return);
}
